class VuexModule {
    constructor(module) {
        this.actions = module.actions;
        this.mutations = module.mutations;
        this.state = module.state;
        this.getters = module.getters;
        this.namespaced = module.namespaced;
        this.modules = module.modules;
    }
}
function getModule(moduleClass, store) {
    if (moduleClass._statics) {
        return moduleClass._statics;
    }
    const genStatic = moduleClass._genStatic;
    if (!genStatic) {
        throw new Error(`ERR_GET_MODULE_NO_STATICS : Could not get module accessor. 
      Make sure your module has name, we can't make accessors for unnamed modules
      i.e. @Module({ 'something' })`);
    }
    return (moduleClass._statics = genStatic(store));
}

const reservedKeys = ['actions', 'getters', 'mutations', 'modules', 'state', 'namespaced', 'commit'];
function stateFactory(module) {
    const state = new module.prototype.constructor({});
    const s = {};
    Object.keys(state).forEach((key) => {
        if (reservedKeys.indexOf(key) !== -1) {
            if (typeof state[key] !== 'undefined') {
                throw new Error(`ERR_RESERVED_STATE_KEY_USED: You cannot use the following
        ['actions', 'getters', 'mutations', 'modules', 'state', 'namespaced', 'commit']
        as fields in your module. These are reserved as they have special purpose in Vuex`);
            }
            return;
        }
        if (state.hasOwnProperty(key)) {
            if (typeof state[key] !== 'function') {
                s[key] = state[key];
            }
        }
    });
    return s;
}

/**
 * Takes the properties on object from parameter source and adds them to the object
 * parameter target
 * @param {object} target  Object to have properties copied onto from y
 * @param {object} source  Object with properties to be copied to x
 */
function addPropertiesToObject(target, source) {
    for (let k of Object.keys(source || {})) {
        Object.defineProperty(target, k, {
            get: () => source[k]
        });
    }
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function staticStateGenerator(module, modOpt, statics) {
    Object.keys(module.state).forEach((key) => {
        if (module.state.hasOwnProperty(key)) {
            // If not undefined or function means it is a state value
            if (['undefined', 'function'].indexOf(typeof module.state[key]) === -1) {
                Object.defineProperty(statics, key, {
                    get() {
                        return modOpt.store.state[modOpt.name][key];
                    }
                });
            }
        }
    });
}
function staticGetterGenerator(module, modOpt, statics) {
    Object.keys(module.getters).forEach((key) => {
        if (module.namespaced) {
            Object.defineProperty(statics, key, {
                get() {
                    return modOpt.store.getters[`${modOpt.name}/${key}`];
                }
            });
        }
        else {
            Object.defineProperty(statics, key, {
                get() {
                    return modOpt.store.getters[key];
                }
            });
        }
    });
}
function staticMutationGenerator(module, modOpt, statics) {
    Object.keys(module.mutations).forEach((key) => {
        if (module.namespaced) {
            statics[key] = function (...args) {
                modOpt.store.commit(`${modOpt.name}/${key}`, ...args);
            };
        }
        else {
            statics[key] = function (...args) {
                modOpt.store.commit(key, ...args);
            };
        }
    });
}
function staticActionGenerators(module, modOpt, statics) {
    Object.keys(module.actions).forEach((key) => {
        if (module.namespaced) {
            statics[key] = function (...args) {
                return __awaiter(this, void 0, void 0, function* () {
                    return modOpt.store.dispatch(`${modOpt.name}/${key}`, ...args);
                });
            };
        }
        else {
            statics[key] = function (...args) {
                return __awaiter(this, void 0, void 0, function* () {
                    return modOpt.store.dispatch(key, ...args);
                });
            };
        }
    });
}

function moduleDecoratorFactory(moduleOptions) {
    return function (constructor) {
        const module = constructor;
        const stateFactory$$1 = () => stateFactory(module);
        if (!module.state) {
            module.state = moduleOptions && moduleOptions.stateFactory ? stateFactory$$1 : stateFactory$$1();
        }
        if (!module.getters) {
            module.getters = {};
        }
        module.namespaced = moduleOptions && moduleOptions.namespaced;
        Object.getOwnPropertyNames(module.prototype).forEach((funcName) => {
            const descriptor = Object.getOwnPropertyDescriptor(module.prototype, funcName);
            if (descriptor.get && module.getters) {
                module.getters[funcName] = function (state, getters, rootState, rootGetters) {
                    const thisObj = { context: { state, getters, rootState, rootGetters } };
                    addPropertiesToObject(thisObj, state);
                    addPropertiesToObject(thisObj, getters);
                    const got = descriptor.get.call(thisObj);
                    return got;
                };
            }
        });
        const modOpt = moduleOptions;
        if (modOpt.name) {
            Object.defineProperty(constructor, '_genStatic', {
                value: (store) => {
                    let statics = {};
                    modOpt.store = modOpt.store || store;
                    if (!modOpt.store) {
                        throw new Error(`ERR_STORE_NOT_PROVIDED: To use getModule(), either the module
            should be decorated with store in decorator, i.e. @Module({store: store}) or
            store should be passed when calling getModule(), i.e. getModule(MyModule, this.$store)`);
                    }
                    // ===========  For statics ==============
                    // ------ state -------
                    staticStateGenerator(module, modOpt, statics);
                    // ------- getters -------
                    if (module.getters) {
                        staticGetterGenerator(module, modOpt, statics);
                    }
                    // -------- mutations --------
                    if (module.mutations) {
                        staticMutationGenerator(module, modOpt, statics);
                    }
                    // -------- actions ---------
                    if (module.actions) {
                        staticActionGenerators(module, modOpt, statics);
                    }
                    return statics;
                }
            });
        }
        if (modOpt.dynamic) {
            if (!modOpt.name) {
                throw new Error('Name of module not provided in decorator options');
            }
            modOpt.store.registerModule(modOpt.name, // TODO: Handle nested modules too in future
            module);
        }
        return constructor;
    };
}
function Module(modOrOpt) {
    if (typeof modOrOpt === 'function') {
        /*
         * @Module decorator called without options (directly on the class definition)
         */
        moduleDecoratorFactory({})(modOrOpt);
    }
    else {
        /*
         * @Module({...}) decorator called with options
         */
        return moduleDecoratorFactory(modOrOpt);
    }
}

function actionDecoratorFactory(params) {
    const { commit = undefined, rawError = false } = params || {};
    return function (target, key, descriptor) {
        const module = target.constructor;
        if (!module.actions) {
            module.actions = {};
        }
        const actionFunction = descriptor.value;
        const action = function (context, payload) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    let actionPayload = null;
                    if (module._genStatic) {
                        const moduleAccessor = getModule(module);
                        moduleAccessor.context = context;
                        actionPayload = yield actionFunction.call(moduleAccessor, payload);
                    }
                    else {
                        const thisObj = { context };
                        addPropertiesToObject(thisObj, context.state);
                        addPropertiesToObject(thisObj, context.getters);
                        actionPayload = yield actionFunction.call(thisObj, payload);
                    }
                    if (commit) {
                        context.commit(commit, actionPayload);
                    }
                    return actionPayload;
                }
                catch (e) {
                    throw rawError
                        ? e
                        : new Error('ERR_ACTION_ACCESS_UNDEFINED: Are you trying to access ' +
                            'this.someMutation() or this.someGetter inside an @Action? \n' +
                            'That works only in dynamic modules. \n' +
                            'If not dynamic use this.context.commit("mutationName", payload) ' +
                            'and this.context.getters["getterName"]' +
                            '\n' +
                            new Error(`Could not perform action ${key.toString()}`).stack +
                            '\n' +
                            e.stack);
                }
            });
        };
        module.actions[key] = action;
    };
}
/**
 * /**
 * The @Action decorator turns an async function into an Vuex action
 *
 * @param targetOrParams the module class
 * @param key name of the action
 * @param descriptor the action function descriptor
 * @constructor
 */
function Action(targetOrParams, key, descriptor) {
    if (!key && !descriptor) {
        /*
         * This is the case when `targetOrParams` is params.
         * i.e. when used as -
         * <pre>
            @Action({commit: 'incrCount'})
            async getCountDelta() {
              return 5
            }
         * </pre>
         */
        return actionDecoratorFactory(targetOrParams);
    }
    else {
        /*
         * This is the case when @Action is called on action function
         * without any params
         * <pre>
         *   @Action
         *   async doSomething() {
         *    ...
         *   }
         * </pre>
         */
        actionDecoratorFactory()(targetOrParams, key, descriptor);
    }
}

function Mutation(target, key, descriptor) {
    const module = target.constructor;
    if (!module.mutations) {
        module.mutations = {};
    }
    const mutationFunction = descriptor.value ? descriptor.value : (...args) => ({});
    const mutation = function (state, payload) {
        mutationFunction.call(state, payload);
    };
    module.mutations[key] = mutation;
}

function MutationAction(params) {
    return function (target, key, descriptor) {
        const module = target.constructor;
        if (!module.mutations) {
            module.mutations = {};
        }
        if (!module.actions) {
            module.actions = {};
        }
        const mutactFunction = descriptor.value;
        const action = function (context, payload) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    const actionPayload = yield mutactFunction.call(context, payload);
                    context.commit(key, actionPayload);
                }
                catch (e) {
                    if (params.rawError) {
                        throw e;
                    }
                    else {
                        console.error('Could not perform action ' + key.toString());
                        console.error(e);
                    }
                }
            });
        };
        const mutation = function (state, payload) {
            for (let stateItem of params.mutate) {
                if (state[stateItem] != null && payload[stateItem] != null) {
                    state[stateItem] = payload[stateItem];
                }
                else {
                    throw new Error(`ERR_MUTATE_PARAMS_NOT_IN_PAYLOAD
          In @MutationAction, mutate: ['a', 'b', ...] array keys must
          match with return type = {a: {}, b: {}, ...} and must
          also be in state.`);
                }
            }
        };
        module.actions[key] = action;
        module.mutations[key] = mutation;
    };
}

export { VuexModule, getModule, Module, Action, Mutation, MutationAction };
//# sourceMappingURL=index.js.map
